using System;
using System.Drawing;
using System.Drawing.Imaging;
using System.Collections;
using System.ComponentModel;
using System.Windows.Forms;
using System.Data;
using System.Threading;
using System.IO; // for streams
using System.Runtime.Serialization;
//using System.Runtime.Serialization.Formatters.Soap;
using System.Runtime.Serialization.Formatters.Binary;

namespace epitome
{
	/// <summary>
	/// Summary description for EpitomeForm.
	/// </summary>
	public class EpitomeForm : System.Windows.Forms.Form
	{
		private System.Windows.Forms.PictureBox InputImage;
		private System.Windows.Forms.Button LoadImage;
		private System.Windows.Forms.Button GenerateEpitome;
		private System.Windows.Forms.Label EpitomeWidthLabel;
		private System.Windows.Forms.Label EpitomeHeightLabel;
		private System.Windows.Forms.Button SaveImage;
		private System.Windows.Forms.PictureBox EpitomeImage;
		private System.Windows.Forms.Label InputImageLabel;
		private System.Windows.Forms.Label EpitomeImageLabel;
		private System.Windows.Forms.Button LoadEpitome;
		private System.Windows.Forms.Button SaveEpitome;
		private System.Windows.Forms.Button RevertImage;
		private System.Windows.Forms.Button MapToImage;
		private System.Windows.Forms.Label PatchSizeLabel;
		private System.Windows.Forms.OpenFileDialog myFileOpenDialog;
		private System.Windows.Forms.SaveFileDialog mySaveFileDialog;
		private System.Windows.Forms.Label InputImagePathLabel;
		private System.Windows.Forms.Label InputImageStatsLabel;
		private System.Windows.Forms.Label EpitomeImageStatsLabel;
		private System.Windows.Forms.Button RevertEpitome;
		private System.Windows.Forms.PictureBox DebugPictureBox;
		private System.Windows.Forms.RichTextBox DebugTextBox;
		private System.Windows.Forms.NumericUpDown EpitomeWidthEntry;
		private System.Windows.Forms.NumericUpDown EpitomeHeightEntry;
		private System.Windows.Forms.NumericUpDown PatchSizeEntry;
		private System.Windows.Forms.NumericUpDown EpitomeIterationsEntry;
		private System.Windows.Forms.CheckBox FixedSizeCheckBox;
		private System.Windows.Forms.Label IterationsLabel;
		private System.Windows.Forms.Button ShowPatches;
		private System.Windows.Forms.Button ShowEpitomeSequence;
		private System.Windows.Forms.Button ShowLatestEpitome;
		private System.Windows.Forms.RadioButton EpitomeShowNormal;
		private System.Windows.Forms.RadioButton EpitomeShowTiled;
		private System.Windows.Forms.RadioButton EpitomeShowScaled;
		private System.Windows.Forms.Label label1;
		private System.Windows.Forms.Label IterationModulusLabel;
		private System.Windows.Forms.NumericUpDown IterationModulusEntry;
		private System.Windows.Forms.Button ShowPatchDensityMap;
		private System.Windows.Forms.Button StopButton;
		private System.Windows.Forms.Label PixelErrorMethodLabel;
		private System.Windows.Forms.ListBox PixelErrorMethodEntry;	
		/// <summary>
		/// Required designer variable.
		/// </summary>
		private System.ComponentModel.Container components = null;

		private Thread WorkerThread;
		private System.Windows.Forms.Label EpitomeFilePathLabel;	// a field for a worker thread
		private EpitomeDocObject EDO;   // a document object containing all the data we actually work with.

		public EpitomeForm()
		{
			//
			// Required for Windows Form Designer support
			//
			InitializeComponent();

			//
			// TODO: Add any constructor code after InitializeComponent call
			//
			EDO = new EpitomeDocObject();     // create a new document object.
			EDO.IsEpitomeDone = false;
			EDO.EpitomeShowState = "normal";
			foreach (string s in Epitome.PixelErrorDelegateLabels) // populate the Pixel Error Methods list.
			{
				PixelErrorMethodEntry.Items.Add(s);
			}
			PixelErrorMethodEntry.SetSelected(0,true); // select the first method in the list
		}

		/// <summary>
		/// Clean up any resources being used.
		/// </summary>
		protected override void Dispose( bool disposing )
		{
			if( disposing )
			{
				if (components != null) 
				{
					components.Dispose();
				}
			}
			base.Dispose( disposing );
		}

		#region Windows Form Designer generated code
		/// <summary>
		/// Required method for Designer support - do not modify
		/// the contents of this method with the code editor.
		/// </summary>
		private void InitializeComponent()
		{
			this.InputImage = new System.Windows.Forms.PictureBox();
			this.LoadImage = new System.Windows.Forms.Button();
			this.InputImagePathLabel = new System.Windows.Forms.Label();
			this.GenerateEpitome = new System.Windows.Forms.Button();
			this.EpitomeWidthLabel = new System.Windows.Forms.Label();
			this.EpitomeHeightLabel = new System.Windows.Forms.Label();
			this.InputImageStatsLabel = new System.Windows.Forms.Label();
			this.SaveImage = new System.Windows.Forms.Button();
			this.EpitomeImage = new System.Windows.Forms.PictureBox();
			this.EpitomeFilePathLabel = new System.Windows.Forms.Label();
			this.EpitomeImageStatsLabel = new System.Windows.Forms.Label();
			this.InputImageLabel = new System.Windows.Forms.Label();
			this.EpitomeImageLabel = new System.Windows.Forms.Label();
			this.LoadEpitome = new System.Windows.Forms.Button();
			this.SaveEpitome = new System.Windows.Forms.Button();
			this.RevertImage = new System.Windows.Forms.Button();
			this.MapToImage = new System.Windows.Forms.Button();
			this.PatchSizeLabel = new System.Windows.Forms.Label();
			this.myFileOpenDialog = new System.Windows.Forms.OpenFileDialog();
			this.mySaveFileDialog = new System.Windows.Forms.SaveFileDialog();
			this.RevertEpitome = new System.Windows.Forms.Button();
			this.DebugTextBox = new System.Windows.Forms.RichTextBox();
			this.DebugPictureBox = new System.Windows.Forms.PictureBox();
			this.EpitomeWidthEntry = new System.Windows.Forms.NumericUpDown();
			this.EpitomeHeightEntry = new System.Windows.Forms.NumericUpDown();
			this.PatchSizeEntry = new System.Windows.Forms.NumericUpDown();
			this.IterationsLabel = new System.Windows.Forms.Label();
			this.EpitomeIterationsEntry = new System.Windows.Forms.NumericUpDown();
			this.FixedSizeCheckBox = new System.Windows.Forms.CheckBox();
			this.ShowPatches = new System.Windows.Forms.Button();
			this.ShowEpitomeSequence = new System.Windows.Forms.Button();
			this.ShowLatestEpitome = new System.Windows.Forms.Button();
			this.EpitomeShowNormal = new System.Windows.Forms.RadioButton();
			this.EpitomeShowTiled = new System.Windows.Forms.RadioButton();
			this.EpitomeShowScaled = new System.Windows.Forms.RadioButton();
			this.label1 = new System.Windows.Forms.Label();
			this.IterationModulusLabel = new System.Windows.Forms.Label();
			this.IterationModulusEntry = new System.Windows.Forms.NumericUpDown();
			this.ShowPatchDensityMap = new System.Windows.Forms.Button();
			this.StopButton = new System.Windows.Forms.Button();
			this.PixelErrorMethodLabel = new System.Windows.Forms.Label();
			this.PixelErrorMethodEntry = new System.Windows.Forms.ListBox();
			((System.ComponentModel.ISupportInitialize)(this.EpitomeWidthEntry)).BeginInit();
			((System.ComponentModel.ISupportInitialize)(this.EpitomeHeightEntry)).BeginInit();
			((System.ComponentModel.ISupportInitialize)(this.PatchSizeEntry)).BeginInit();
			((System.ComponentModel.ISupportInitialize)(this.EpitomeIterationsEntry)).BeginInit();
			((System.ComponentModel.ISupportInitialize)(this.IterationModulusEntry)).BeginInit();
			this.SuspendLayout();
			// 
			// InputImage
			// 
			this.InputImage.Anchor = ((System.Windows.Forms.AnchorStyles)((((System.Windows.Forms.AnchorStyles.Top | System.Windows.Forms.AnchorStyles.Bottom) 
				| System.Windows.Forms.AnchorStyles.Left) 
				| System.Windows.Forms.AnchorStyles.Right)));
			this.InputImage.BorderStyle = System.Windows.Forms.BorderStyle.FixedSingle;
			this.InputImage.Location = new System.Drawing.Point(8, 24);
			this.InputImage.Name = "InputImage";
			this.InputImage.Size = new System.Drawing.Size(448, 320);
			this.InputImage.TabIndex = 1;
			this.InputImage.TabStop = false;
			this.InputImage.MouseDown += new System.Windows.Forms.MouseEventHandler(this.InputImage_MouseDown);
			// 
			// LoadImage
			// 
			this.LoadImage.Anchor = ((System.Windows.Forms.AnchorStyles)((System.Windows.Forms.AnchorStyles.Bottom | System.Windows.Forms.AnchorStyles.Left)));
			this.LoadImage.Location = new System.Drawing.Point(8, 400);
			this.LoadImage.Name = "LoadImage";
			this.LoadImage.Size = new System.Drawing.Size(72, 23);
			this.LoadImage.TabIndex = 2;
			this.LoadImage.Text = "Load Image";
			this.LoadImage.Click += new System.EventHandler(this.LoadImage_Click);
			// 
			// InputImagePathLabel
			// 
			this.InputImagePathLabel.Anchor = ((System.Windows.Forms.AnchorStyles)((System.Windows.Forms.AnchorStyles.Bottom | System.Windows.Forms.AnchorStyles.Left)));
			this.InputImagePathLabel.BorderStyle = System.Windows.Forms.BorderStyle.FixedSingle;
			this.InputImagePathLabel.Location = new System.Drawing.Point(8, 352);
			this.InputImagePathLabel.Name = "InputImagePathLabel";
			this.InputImagePathLabel.Size = new System.Drawing.Size(448, 16);
			this.InputImagePathLabel.TabIndex = 3;
			// 
			// GenerateEpitome
			// 
			this.GenerateEpitome.Anchor = ((System.Windows.Forms.AnchorStyles)((System.Windows.Forms.AnchorStyles.Top | System.Windows.Forms.AnchorStyles.Right)));
			this.GenerateEpitome.Enabled = false;
			this.GenerateEpitome.Location = new System.Drawing.Point(472, 272);
			this.GenerateEpitome.Name = "GenerateEpitome";
			this.GenerateEpitome.Size = new System.Drawing.Size(96, 32);
			this.GenerateEpitome.TabIndex = 4;
			this.GenerateEpitome.Text = "Generate Epitome";
			this.GenerateEpitome.Click += new System.EventHandler(this.GenerateEpitome_Click);
			// 
			// EpitomeWidthLabel
			// 
			this.EpitomeWidthLabel.Anchor = ((System.Windows.Forms.AnchorStyles)((System.Windows.Forms.AnchorStyles.Top | System.Windows.Forms.AnchorStyles.Right)));
			this.EpitomeWidthLabel.Location = new System.Drawing.Point(472, 8);
			this.EpitomeWidthLabel.Name = "EpitomeWidthLabel";
			this.EpitomeWidthLabel.Size = new System.Drawing.Size(72, 16);
			this.EpitomeWidthLabel.TabIndex = 6;
			this.EpitomeWidthLabel.Text = "Width:";
			// 
			// EpitomeHeightLabel
			// 
			this.EpitomeHeightLabel.Anchor = ((System.Windows.Forms.AnchorStyles)((System.Windows.Forms.AnchorStyles.Top | System.Windows.Forms.AnchorStyles.Right)));
			this.EpitomeHeightLabel.Location = new System.Drawing.Point(472, 48);
			this.EpitomeHeightLabel.Name = "EpitomeHeightLabel";
			this.EpitomeHeightLabel.Size = new System.Drawing.Size(100, 16);
			this.EpitomeHeightLabel.TabIndex = 7;
			this.EpitomeHeightLabel.Text = "Height:";
			// 
			// InputImageStatsLabel
			// 
			this.InputImageStatsLabel.Anchor = ((System.Windows.Forms.AnchorStyles)((System.Windows.Forms.AnchorStyles.Bottom | System.Windows.Forms.AnchorStyles.Left)));
			this.InputImageStatsLabel.BorderStyle = System.Windows.Forms.BorderStyle.FixedSingle;
			this.InputImageStatsLabel.Location = new System.Drawing.Point(8, 376);
			this.InputImageStatsLabel.Name = "InputImageStatsLabel";
			this.InputImageStatsLabel.Size = new System.Drawing.Size(448, 16);
			this.InputImageStatsLabel.TabIndex = 10;
			// 
			// SaveImage
			// 
			this.SaveImage.Anchor = ((System.Windows.Forms.AnchorStyles)((System.Windows.Forms.AnchorStyles.Bottom | System.Windows.Forms.AnchorStyles.Left)));
			this.SaveImage.Location = new System.Drawing.Point(88, 400);
			this.SaveImage.Name = "SaveImage";
			this.SaveImage.Size = new System.Drawing.Size(72, 23);
			this.SaveImage.TabIndex = 11;
			this.SaveImage.Text = "Save Image";
			this.SaveImage.Click += new System.EventHandler(this.SaveImage_Click);
			// 
			// EpitomeImage
			// 
			this.EpitomeImage.Anchor = ((System.Windows.Forms.AnchorStyles)((System.Windows.Forms.AnchorStyles.Top | System.Windows.Forms.AnchorStyles.Right)));
			this.EpitomeImage.BorderStyle = System.Windows.Forms.BorderStyle.FixedSingle;
			this.EpitomeImage.Location = new System.Drawing.Point(584, 24);
			this.EpitomeImage.Name = "EpitomeImage";
			this.EpitomeImage.Size = new System.Drawing.Size(392, 320);
			this.EpitomeImage.TabIndex = 12;
			this.EpitomeImage.TabStop = false;
			this.EpitomeImage.MouseDown += new System.Windows.Forms.MouseEventHandler(this.EpitomeImage_MouseDown);
			// 
			// EpitomeFilePathLabel
			// 
			this.EpitomeFilePathLabel.Anchor = ((System.Windows.Forms.AnchorStyles)((System.Windows.Forms.AnchorStyles.Top | System.Windows.Forms.AnchorStyles.Right)));
			this.EpitomeFilePathLabel.BorderStyle = System.Windows.Forms.BorderStyle.FixedSingle;
			this.EpitomeFilePathLabel.Location = new System.Drawing.Point(584, 352);
			this.EpitomeFilePathLabel.Name = "EpitomeFilePathLabel";
			this.EpitomeFilePathLabel.Size = new System.Drawing.Size(392, 16);
			this.EpitomeFilePathLabel.TabIndex = 13;
			// 
			// EpitomeImageStatsLabel
			// 
			this.EpitomeImageStatsLabel.Anchor = ((System.Windows.Forms.AnchorStyles)((System.Windows.Forms.AnchorStyles.Top | System.Windows.Forms.AnchorStyles.Right)));
			this.EpitomeImageStatsLabel.BorderStyle = System.Windows.Forms.BorderStyle.FixedSingle;
			this.EpitomeImageStatsLabel.Location = new System.Drawing.Point(584, 376);
			this.EpitomeImageStatsLabel.Name = "EpitomeImageStatsLabel";
			this.EpitomeImageStatsLabel.Size = new System.Drawing.Size(392, 16);
			this.EpitomeImageStatsLabel.TabIndex = 14;
			// 
			// InputImageLabel
			// 
			this.InputImageLabel.Location = new System.Drawing.Point(8, 8);
			this.InputImageLabel.Name = "InputImageLabel";
			this.InputImageLabel.Size = new System.Drawing.Size(64, 16);
			this.InputImageLabel.TabIndex = 15;
			this.InputImageLabel.Text = "Input Image";
			// 
			// EpitomeImageLabel
			// 
			this.EpitomeImageLabel.Anchor = ((System.Windows.Forms.AnchorStyles)((System.Windows.Forms.AnchorStyles.Top | System.Windows.Forms.AnchorStyles.Right)));
			this.EpitomeImageLabel.Location = new System.Drawing.Point(584, 8);
			this.EpitomeImageLabel.Name = "EpitomeImageLabel";
			this.EpitomeImageLabel.Size = new System.Drawing.Size(48, 16);
			this.EpitomeImageLabel.TabIndex = 16;
			this.EpitomeImageLabel.Text = "Epitome";
			// 
			// LoadEpitome
			// 
			this.LoadEpitome.Anchor = ((System.Windows.Forms.AnchorStyles)((System.Windows.Forms.AnchorStyles.Top | System.Windows.Forms.AnchorStyles.Right)));
			this.LoadEpitome.Location = new System.Drawing.Point(584, 400);
			this.LoadEpitome.Name = "LoadEpitome";
			this.LoadEpitome.Size = new System.Drawing.Size(88, 23);
			this.LoadEpitome.TabIndex = 17;
			this.LoadEpitome.Text = "Load Epitome";
			this.LoadEpitome.Click += new System.EventHandler(this.LoadEpitome_Click);
			// 
			// SaveEpitome
			// 
			this.SaveEpitome.Anchor = ((System.Windows.Forms.AnchorStyles)((System.Windows.Forms.AnchorStyles.Top | System.Windows.Forms.AnchorStyles.Right)));
			this.SaveEpitome.Enabled = false;
			this.SaveEpitome.Location = new System.Drawing.Point(680, 400);
			this.SaveEpitome.Name = "SaveEpitome";
			this.SaveEpitome.Size = new System.Drawing.Size(88, 23);
			this.SaveEpitome.TabIndex = 18;
			this.SaveEpitome.Text = "Save Epitome";
			this.SaveEpitome.Click += new System.EventHandler(this.SaveEpitome_Click);
			// 
			// RevertImage
			// 
			this.RevertImage.Anchor = ((System.Windows.Forms.AnchorStyles)((System.Windows.Forms.AnchorStyles.Bottom | System.Windows.Forms.AnchorStyles.Left)));
			this.RevertImage.Enabled = false;
			this.RevertImage.Location = new System.Drawing.Point(168, 400);
			this.RevertImage.Name = "RevertImage";
			this.RevertImage.TabIndex = 19;
			this.RevertImage.Text = "Revert";
			this.RevertImage.Click += new System.EventHandler(this.RevertImage_Click);
			// 
			// MapToImage
			// 
			this.MapToImage.Anchor = ((System.Windows.Forms.AnchorStyles)((System.Windows.Forms.AnchorStyles.Top | System.Windows.Forms.AnchorStyles.Right)));
			this.MapToImage.Enabled = false;
			this.MapToImage.Location = new System.Drawing.Point(472, 344);
			this.MapToImage.Name = "MapToImage";
			this.MapToImage.Size = new System.Drawing.Size(96, 32);
			this.MapToImage.TabIndex = 20;
			this.MapToImage.Text = "Map to Image";
			this.MapToImage.Click += new System.EventHandler(this.MapToImage_Click);
			// 
			// PatchSizeLabel
			// 
			this.PatchSizeLabel.Anchor = ((System.Windows.Forms.AnchorStyles)((System.Windows.Forms.AnchorStyles.Top | System.Windows.Forms.AnchorStyles.Right)));
			this.PatchSizeLabel.Location = new System.Drawing.Point(472, 88);
			this.PatchSizeLabel.Name = "PatchSizeLabel";
			this.PatchSizeLabel.Size = new System.Drawing.Size(100, 16);
			this.PatchSizeLabel.TabIndex = 21;
			this.PatchSizeLabel.Text = "Patch Size";
			// 
			// RevertEpitome
			// 
			this.RevertEpitome.Anchor = ((System.Windows.Forms.AnchorStyles)((System.Windows.Forms.AnchorStyles.Top | System.Windows.Forms.AnchorStyles.Right)));
			this.RevertEpitome.Location = new System.Drawing.Point(776, 400);
			this.RevertEpitome.Name = "RevertEpitome";
			this.RevertEpitome.TabIndex = 23;
			this.RevertEpitome.Text = "Revert";
			// 
			// DebugTextBox
			// 
			this.DebugTextBox.Anchor = ((System.Windows.Forms.AnchorStyles)(((System.Windows.Forms.AnchorStyles.Bottom | System.Windows.Forms.AnchorStyles.Left) 
				| System.Windows.Forms.AnchorStyles.Right)));
			this.DebugTextBox.BorderStyle = System.Windows.Forms.BorderStyle.FixedSingle;
			this.DebugTextBox.CausesValidation = false;
			this.DebugTextBox.Font = new System.Drawing.Font("Courier New", 9.75F, System.Drawing.FontStyle.Regular, System.Drawing.GraphicsUnit.Point, ((System.Byte)(0)));
			this.DebugTextBox.Location = new System.Drawing.Point(8, 432);
			this.DebugTextBox.Name = "DebugTextBox";
			this.DebugTextBox.Size = new System.Drawing.Size(448, 312);
			this.DebugTextBox.TabIndex = 24;
			this.DebugTextBox.Text = "";
			// 
			// DebugPictureBox
			// 
			this.DebugPictureBox.Anchor = ((System.Windows.Forms.AnchorStyles)(((System.Windows.Forms.AnchorStyles.Top | System.Windows.Forms.AnchorStyles.Bottom) 
				| System.Windows.Forms.AnchorStyles.Right)));
			this.DebugPictureBox.BorderStyle = System.Windows.Forms.BorderStyle.FixedSingle;
			this.DebugPictureBox.Location = new System.Drawing.Point(584, 432);
			this.DebugPictureBox.Name = "DebugPictureBox";
			this.DebugPictureBox.Size = new System.Drawing.Size(400, 312);
			this.DebugPictureBox.TabIndex = 25;
			this.DebugPictureBox.TabStop = false;
			// 
			// EpitomeWidthEntry
			// 
			this.EpitomeWidthEntry.Anchor = ((System.Windows.Forms.AnchorStyles)((System.Windows.Forms.AnchorStyles.Top | System.Windows.Forms.AnchorStyles.Right)));
			this.EpitomeWidthEntry.Location = new System.Drawing.Point(472, 24);
			this.EpitomeWidthEntry.Maximum = new System.Decimal(new int[] {
																			  1000,
																			  0,
																			  0,
																			  0});
			this.EpitomeWidthEntry.Minimum = new System.Decimal(new int[] {
																			  10,
																			  0,
																			  0,
																			  0});
			this.EpitomeWidthEntry.Name = "EpitomeWidthEntry";
			this.EpitomeWidthEntry.Size = new System.Drawing.Size(104, 20);
			this.EpitomeWidthEntry.TabIndex = 26;
			this.EpitomeWidthEntry.Value = new System.Decimal(new int[] {
																			256,
																			0,
																			0,
																			0});
			// 
			// EpitomeHeightEntry
			// 
			this.EpitomeHeightEntry.Anchor = ((System.Windows.Forms.AnchorStyles)((System.Windows.Forms.AnchorStyles.Top | System.Windows.Forms.AnchorStyles.Right)));
			this.EpitomeHeightEntry.Location = new System.Drawing.Point(472, 64);
			this.EpitomeHeightEntry.Maximum = new System.Decimal(new int[] {
																			   1000,
																			   0,
																			   0,
																			   0});
			this.EpitomeHeightEntry.Minimum = new System.Decimal(new int[] {
																			   10,
																			   0,
																			   0,
																			   0});
			this.EpitomeHeightEntry.Name = "EpitomeHeightEntry";
			this.EpitomeHeightEntry.Size = new System.Drawing.Size(104, 20);
			this.EpitomeHeightEntry.TabIndex = 27;
			this.EpitomeHeightEntry.Value = new System.Decimal(new int[] {
																			 256,
																			 0,
																			 0,
																			 0});
			// 
			// PatchSizeEntry
			// 
			this.PatchSizeEntry.Anchor = ((System.Windows.Forms.AnchorStyles)((System.Windows.Forms.AnchorStyles.Top | System.Windows.Forms.AnchorStyles.Right)));
			this.PatchSizeEntry.Location = new System.Drawing.Point(472, 104);
			this.PatchSizeEntry.Maximum = new System.Decimal(new int[] {
																		   32,
																		   0,
																		   0,
																		   0});
			this.PatchSizeEntry.Minimum = new System.Decimal(new int[] {
																		   2,
																		   0,
																		   0,
																		   0});
			this.PatchSizeEntry.Name = "PatchSizeEntry";
			this.PatchSizeEntry.Size = new System.Drawing.Size(104, 20);
			this.PatchSizeEntry.TabIndex = 28;
			this.PatchSizeEntry.Value = new System.Decimal(new int[] {
																		 12,
																		 0,
																		 0,
																		 0});
			// 
			// IterationsLabel
			// 
			this.IterationsLabel.Anchor = ((System.Windows.Forms.AnchorStyles)((System.Windows.Forms.AnchorStyles.Top | System.Windows.Forms.AnchorStyles.Right)));
			this.IterationsLabel.Location = new System.Drawing.Point(472, 152);
			this.IterationsLabel.Name = "IterationsLabel";
			this.IterationsLabel.Size = new System.Drawing.Size(100, 16);
			this.IterationsLabel.TabIndex = 29;
			this.IterationsLabel.Text = "Iterations:";
			// 
			// EpitomeIterationsEntry
			// 
			this.EpitomeIterationsEntry.Anchor = ((System.Windows.Forms.AnchorStyles)((System.Windows.Forms.AnchorStyles.Top | System.Windows.Forms.AnchorStyles.Right)));
			this.EpitomeIterationsEntry.Location = new System.Drawing.Point(472, 168);
			this.EpitomeIterationsEntry.Maximum = new System.Decimal(new int[] {
																				   1000,
																				   0,
																				   0,
																				   0});
			this.EpitomeIterationsEntry.Name = "EpitomeIterationsEntry";
			this.EpitomeIterationsEntry.Size = new System.Drawing.Size(104, 20);
			this.EpitomeIterationsEntry.TabIndex = 30;
			this.EpitomeIterationsEntry.Value = new System.Decimal(new int[] {
																				 2,
																				 0,
																				 0,
																				 0});
			// 
			// FixedSizeCheckBox
			// 
			this.FixedSizeCheckBox.Anchor = ((System.Windows.Forms.AnchorStyles)((System.Windows.Forms.AnchorStyles.Top | System.Windows.Forms.AnchorStyles.Right)));
			this.FixedSizeCheckBox.Checked = true;
			this.FixedSizeCheckBox.CheckState = System.Windows.Forms.CheckState.Checked;
			this.FixedSizeCheckBox.Location = new System.Drawing.Point(472, 128);
			this.FixedSizeCheckBox.Name = "FixedSizeCheckBox";
			this.FixedSizeCheckBox.Size = new System.Drawing.Size(104, 16);
			this.FixedSizeCheckBox.TabIndex = 32;
			this.FixedSizeCheckBox.Text = "Fixed Size";
			// 
			// ShowPatches
			// 
			this.ShowPatches.Anchor = ((System.Windows.Forms.AnchorStyles)((System.Windows.Forms.AnchorStyles.Top | System.Windows.Forms.AnchorStyles.Right)));
			this.ShowPatches.Enabled = false;
			this.ShowPatches.Location = new System.Drawing.Point(464, 432);
			this.ShowPatches.Name = "ShowPatches";
			this.ShowPatches.Size = new System.Drawing.Size(112, 23);
			this.ShowPatches.TabIndex = 33;
			this.ShowPatches.Text = "Patches";
			this.ShowPatches.Click += new System.EventHandler(this.ShowPatches_Click);
			// 
			// ShowEpitomeSequence
			// 
			this.ShowEpitomeSequence.Anchor = ((System.Windows.Forms.AnchorStyles)((System.Windows.Forms.AnchorStyles.Top | System.Windows.Forms.AnchorStyles.Right)));
			this.ShowEpitomeSequence.Enabled = false;
			this.ShowEpitomeSequence.Location = new System.Drawing.Point(464, 496);
			this.ShowEpitomeSequence.Name = "ShowEpitomeSequence";
			this.ShowEpitomeSequence.Size = new System.Drawing.Size(112, 23);
			this.ShowEpitomeSequence.TabIndex = 34;
			this.ShowEpitomeSequence.Text = "Epitome Sequence";
			this.ShowEpitomeSequence.Click += new System.EventHandler(this.ShowEpitomeSequence_Click);
			// 
			// ShowLatestEpitome
			// 
			this.ShowLatestEpitome.Anchor = ((System.Windows.Forms.AnchorStyles)((System.Windows.Forms.AnchorStyles.Top | System.Windows.Forms.AnchorStyles.Right)));
			this.ShowLatestEpitome.Enabled = false;
			this.ShowLatestEpitome.Location = new System.Drawing.Point(464, 464);
			this.ShowLatestEpitome.Name = "ShowLatestEpitome";
			this.ShowLatestEpitome.Size = new System.Drawing.Size(112, 23);
			this.ShowLatestEpitome.TabIndex = 35;
			this.ShowLatestEpitome.Text = "Latest Epitome";
			this.ShowLatestEpitome.Click += new System.EventHandler(this.ShowLatestEpitome_Click);
			// 
			// EpitomeShowNormal
			// 
			this.EpitomeShowNormal.Anchor = ((System.Windows.Forms.AnchorStyles)((System.Windows.Forms.AnchorStyles.Top | System.Windows.Forms.AnchorStyles.Right)));
			this.EpitomeShowNormal.Checked = true;
			this.EpitomeShowNormal.Location = new System.Drawing.Point(792, 8);
			this.EpitomeShowNormal.Name = "EpitomeShowNormal";
			this.EpitomeShowNormal.Size = new System.Drawing.Size(64, 16);
			this.EpitomeShowNormal.TabIndex = 36;
			this.EpitomeShowNormal.TabStop = true;
			this.EpitomeShowNormal.Text = "Normal";
			this.EpitomeShowNormal.CheckedChanged += new System.EventHandler(this.EpitomeShowNormal_CheckedChanged);
			// 
			// EpitomeShowTiled
			// 
			this.EpitomeShowTiled.Anchor = ((System.Windows.Forms.AnchorStyles)((System.Windows.Forms.AnchorStyles.Top | System.Windows.Forms.AnchorStyles.Right)));
			this.EpitomeShowTiled.Location = new System.Drawing.Point(856, 8);
			this.EpitomeShowTiled.Name = "EpitomeShowTiled";
			this.EpitomeShowTiled.Size = new System.Drawing.Size(48, 16);
			this.EpitomeShowTiled.TabIndex = 37;
			this.EpitomeShowTiled.Text = "Tiled";
			this.EpitomeShowTiled.CheckedChanged += new System.EventHandler(this.EpitomeShowTiled_CheckedChanged);
			// 
			// EpitomeShowScaled
			// 
			this.EpitomeShowScaled.Anchor = ((System.Windows.Forms.AnchorStyles)((System.Windows.Forms.AnchorStyles.Top | System.Windows.Forms.AnchorStyles.Right)));
			this.EpitomeShowScaled.Location = new System.Drawing.Point(912, 8);
			this.EpitomeShowScaled.Name = "EpitomeShowScaled";
			this.EpitomeShowScaled.Size = new System.Drawing.Size(64, 16);
			this.EpitomeShowScaled.TabIndex = 38;
			this.EpitomeShowScaled.Text = "Scaled";
			this.EpitomeShowScaled.CheckedChanged += new System.EventHandler(this.EpitomeShowScaled_CheckedChanged);
			// 
			// label1
			// 
			this.label1.Anchor = ((System.Windows.Forms.AnchorStyles)((System.Windows.Forms.AnchorStyles.Top | System.Windows.Forms.AnchorStyles.Right)));
			this.label1.Location = new System.Drawing.Point(752, 8);
			this.label1.Name = "label1";
			this.label1.Size = new System.Drawing.Size(40, 16);
			this.label1.TabIndex = 39;
			this.label1.Text = "Show:";
			// 
			// IterationModulusLabel
			// 
			this.IterationModulusLabel.Anchor = ((System.Windows.Forms.AnchorStyles)((System.Windows.Forms.AnchorStyles.Top | System.Windows.Forms.AnchorStyles.Right)));
			this.IterationModulusLabel.Location = new System.Drawing.Point(472, 192);
			this.IterationModulusLabel.Name = "IterationModulusLabel";
			this.IterationModulusLabel.Size = new System.Drawing.Size(100, 16);
			this.IterationModulusLabel.TabIndex = 40;
			this.IterationModulusLabel.Text = "Iteration Modulus";
			// 
			// IterationModulusEntry
			// 
			this.IterationModulusEntry.Anchor = ((System.Windows.Forms.AnchorStyles)((System.Windows.Forms.AnchorStyles.Top | System.Windows.Forms.AnchorStyles.Right)));
			this.IterationModulusEntry.Location = new System.Drawing.Point(472, 208);
			this.IterationModulusEntry.Minimum = new System.Decimal(new int[] {
																				  1,
																				  0,
																				  0,
																				  0});
			this.IterationModulusEntry.Name = "IterationModulusEntry";
			this.IterationModulusEntry.Size = new System.Drawing.Size(104, 20);
			this.IterationModulusEntry.TabIndex = 41;
			this.IterationModulusEntry.Value = new System.Decimal(new int[] {
																				3,
																				0,
																				0,
																				0});
			// 
			// ShowPatchDensityMap
			// 
			this.ShowPatchDensityMap.Anchor = ((System.Windows.Forms.AnchorStyles)((System.Windows.Forms.AnchorStyles.Top | System.Windows.Forms.AnchorStyles.Right)));
			this.ShowPatchDensityMap.Enabled = false;
			this.ShowPatchDensityMap.Location = new System.Drawing.Point(464, 528);
			this.ShowPatchDensityMap.Name = "ShowPatchDensityMap";
			this.ShowPatchDensityMap.Size = new System.Drawing.Size(112, 23);
			this.ShowPatchDensityMap.TabIndex = 42;
			this.ShowPatchDensityMap.Text = "Patch Density Map";
			this.ShowPatchDensityMap.Click += new System.EventHandler(this.ShowPatchDensityMap_Click);
			// 
			// StopButton
			// 
			this.StopButton.Anchor = ((System.Windows.Forms.AnchorStyles)((System.Windows.Forms.AnchorStyles.Top | System.Windows.Forms.AnchorStyles.Right)));
			this.StopButton.Enabled = false;
			this.StopButton.Location = new System.Drawing.Point(472, 312);
			this.StopButton.Name = "StopButton";
			this.StopButton.Size = new System.Drawing.Size(96, 23);
			this.StopButton.TabIndex = 43;
			this.StopButton.Text = "Stop";
			this.StopButton.Click += new System.EventHandler(this.StopButton_Click);
			// 
			// PixelErrorMethodLabel
			// 
			this.PixelErrorMethodLabel.Anchor = ((System.Windows.Forms.AnchorStyles)((System.Windows.Forms.AnchorStyles.Top | System.Windows.Forms.AnchorStyles.Right)));
			this.PixelErrorMethodLabel.Location = new System.Drawing.Point(472, 232);
			this.PixelErrorMethodLabel.Name = "PixelErrorMethodLabel";
			this.PixelErrorMethodLabel.Size = new System.Drawing.Size(100, 16);
			this.PixelErrorMethodLabel.TabIndex = 44;
			this.PixelErrorMethodLabel.Text = "Pixel Error Method";
			// 
			// PixelErrorMethodEntry
			// 
			this.PixelErrorMethodEntry.Anchor = ((System.Windows.Forms.AnchorStyles)((System.Windows.Forms.AnchorStyles.Top | System.Windows.Forms.AnchorStyles.Right)));
			this.PixelErrorMethodEntry.Location = new System.Drawing.Point(472, 248);
			this.PixelErrorMethodEntry.Name = "PixelErrorMethodEntry";
			this.PixelErrorMethodEntry.Size = new System.Drawing.Size(104, 17);
			this.PixelErrorMethodEntry.TabIndex = 45;
			// 
			// EpitomeForm
			// 
			this.AutoScaleBaseSize = new System.Drawing.Size(5, 13);
			this.ClientSize = new System.Drawing.Size(984, 746);
			this.Controls.Add(this.PixelErrorMethodEntry);
			this.Controls.Add(this.PixelErrorMethodLabel);
			this.Controls.Add(this.StopButton);
			this.Controls.Add(this.ShowPatchDensityMap);
			this.Controls.Add(this.IterationModulusEntry);
			this.Controls.Add(this.IterationModulusLabel);
			this.Controls.Add(this.label1);
			this.Controls.Add(this.EpitomeShowScaled);
			this.Controls.Add(this.EpitomeShowTiled);
			this.Controls.Add(this.EpitomeShowNormal);
			this.Controls.Add(this.ShowLatestEpitome);
			this.Controls.Add(this.ShowEpitomeSequence);
			this.Controls.Add(this.ShowPatches);
			this.Controls.Add(this.FixedSizeCheckBox);
			this.Controls.Add(this.EpitomeIterationsEntry);
			this.Controls.Add(this.IterationsLabel);
			this.Controls.Add(this.PatchSizeEntry);
			this.Controls.Add(this.EpitomeHeightEntry);
			this.Controls.Add(this.EpitomeWidthEntry);
			this.Controls.Add(this.DebugPictureBox);
			this.Controls.Add(this.DebugTextBox);
			this.Controls.Add(this.RevertEpitome);
			this.Controls.Add(this.PatchSizeLabel);
			this.Controls.Add(this.MapToImage);
			this.Controls.Add(this.RevertImage);
			this.Controls.Add(this.SaveEpitome);
			this.Controls.Add(this.LoadEpitome);
			this.Controls.Add(this.EpitomeImageLabel);
			this.Controls.Add(this.InputImageLabel);
			this.Controls.Add(this.EpitomeImageStatsLabel);
			this.Controls.Add(this.EpitomeFilePathLabel);
			this.Controls.Add(this.EpitomeImage);
			this.Controls.Add(this.SaveImage);
			this.Controls.Add(this.InputImageStatsLabel);
			this.Controls.Add(this.EpitomeHeightLabel);
			this.Controls.Add(this.EpitomeWidthLabel);
			this.Controls.Add(this.GenerateEpitome);
			this.Controls.Add(this.InputImagePathLabel);
			this.Controls.Add(this.LoadImage);
			this.Controls.Add(this.InputImage);
			this.Name = "EpitomeForm";
			this.Text = "Epitome";
			this.Resize += new System.EventHandler(this.OnResize);
			this.Load += new System.EventHandler(this.EpitomeForm_Load);
			((System.ComponentModel.ISupportInitialize)(this.EpitomeWidthEntry)).EndInit();
			((System.ComponentModel.ISupportInitialize)(this.EpitomeHeightEntry)).EndInit();
			((System.ComponentModel.ISupportInitialize)(this.PatchSizeEntry)).EndInit();
			((System.ComponentModel.ISupportInitialize)(this.EpitomeIterationsEntry)).EndInit();
			((System.ComponentModel.ISupportInitialize)(this.IterationModulusEntry)).EndInit();
			this.ResumeLayout(false);

		}
		#endregion

		/// <summary>
		/// The main entry point for the application.
		/// </summary>
		[STAThread]
		static void Main() 
		{
			Application.Run(new EpitomeForm());
		}

		private void EpitomeForm_Load(object sender, System.EventArgs e)
		{
				myFileOpenDialog.Filter = 
					"Image Files (bmp, emf, exif, gif, ico, jpg, png, tiff, wmf)|" +
					"*.bmp;*.emf;*.exif;*.ico;*.jpg;*.jpeg;*.png;*.tiff;*.wmf|" +
					"All Files (*.*)|*.*";
				mySaveFileDialog.Filter = 
					"Image Files (bmp, emf, exif, gif, ico, jpg, png, tiff, wmf)|" +
					"*.bmp;*.emf;*.exif;*.ico;*.jpg;*.jpeg;*.png;*.tiff;*.wmf|" +
					"All Files (*.*)|*.*";
		}

		private void LoadImage_Click(object sender, System.EventArgs e)
		{
			// invoke file-open dialog
			// load the image thus picked into the InputImage object.
			if(myFileOpenDialog.ShowDialog() == DialogResult.OK) 
			{
				EDO.InputImagePath = myFileOpenDialog.FileName;
				EDO.InputImageBitmap = new Bitmap(EDO.InputImagePath);   // load the input image
				ShowInputImage();										 // also called by OnResize();

				InputImagePathLabel.Text = EDO.InputImagePath;	// show the path name in the UI.

				int EW, EH;										// set the epitome size controls to a default of 1/16th
				EW = EDO.InputImageBitmap.Width  / 4;			// of the input image size
				EH = EDO.InputImageBitmap.Height / 4;
				if(EW > 256 || EH > 256)						// unless that would be huge, in which case, use a scale
				{												// that gives a maximum Epitome dimension of 256.  User 
					double Wscale = EDO.InputImageBitmap.Width / 256.0;  // can still override.
					double Hscale = EDO.InputImageBitmap.Height / 256.0;
					double scale = Math.Max(Wscale, Hscale);
					EW = (int)(EDO.InputImageBitmap.Width / scale);
					EH = (int)(EDO.InputImageBitmap.Height / scale);
				}
				EpitomeWidthEntry.Value = EW;
				EpitomeHeightEntry.Value = EH;

				// and finally, enable the generate epitome button:
				GenerateEpitome.Enabled = true;
				RevertImage.Enabled = false;
			}
		}

		/// <summary>
		/// Display the contents of EDO.InputImageBitmap in the InputImage picturebox.  This method takes into
		/// account the relative sizes of the two, and sets the pictureboxsizemode accordingly.  This method
		/// is called when the user loads an image file, or during the form's resize processing.
		/// </summary>
		private void ShowInputImage() 
		{
			if(EDO.InputImageBitmap.Width > InputImage.Width ||  // if input image is too big, then make the picturebox
				EDO.InputImageBitmap.Height > InputImage.Height) // scale it to fit.
				InputImage.SizeMode   = PictureBoxSizeMode.StretchImage;
			else
				InputImage.SizeMode   = PictureBoxSizeMode.Normal;  // reset in case we've loaded multiple images.

			InputImage.Image = (Image)EDO.InputImageBitmap;     // put input image in the picturebox

			InputImagePathLabel.Text = EDO.InputImagePath;  // show the image's path in the UI
			InputImageStatsLabel.Text = "Image Size: " +    // show the image's attributes in the UI
				EDO.InputImageBitmap.Width.ToString() + " x " + 
				EDO.InputImageBitmap.Height.ToString();
		}

		/// <summary>
		/// Uses regular expressions to get the extension of a filename, which is used to map to an image format
		/// from the Imaging.ImageFormat class.  How irritating that I have to write this function at all.
		/// </summary>
		/// <param name="filename"></param>
		/// <returns></returns>
		private ImageFormat GetFormatFromFilename(string filename) 
		{
			string[] substrings = filename.Split(new char[]{'.'});
			string extension = substrings[substrings.GetUpperBound(0)];
			extension = extension.ToLower();

			switch(extension)
			{
				case "bmp":
					return ImageFormat.Bmp;
				case "emf":
					return ImageFormat.Emf;
				case "exif":
					return ImageFormat.Exif;
				case "gif":
					return ImageFormat.Gif;
				case "ico":
					return ImageFormat.Icon;
				case "jpg":
				case "jpeg":
					return ImageFormat.Jpeg;
				case "png":
					return ImageFormat.Png;
				case "tiff":
					return ImageFormat.Tiff;
				case "wmf":
					return ImageFormat.Wmf;
				default:
					// Show a warning message error message and return the PNG case.
					MessageBox.Show("Unknown image format: " + extension + "\nSaving in PNG format", "Unknown Format",
						MessageBoxButtons.OK, MessageBoxIcon.Information);
					goto case "png";
			}
		}
		
		/// <summary>
		/// Save whatever is currently in the InputImage control to a file.
		/// </summary>
		/// <param name="sender"></param>
		/// <param name="e"></param>
		private void SaveImage_Click(object sender, System.EventArgs e)
		{
			if(mySaveFileDialog.ShowDialog() == DialogResult.OK) 
			{
				InputImage.Image.Save(mySaveFileDialog.FileName, GetFormatFromFilename(mySaveFileDialog.FileName));
			}
		}
		/// <summary>
		/// Save a working set--an EpitomeDocObject--to a ".epi" file.
		/// </summary>
		/// <param name="sender"></param>
		/// <param name="e"></param>
		private void SaveEpitome_Click(object sender, System.EventArgs e)
		{
			string filtercache = mySaveFileDialog.Filter;
			mySaveFileDialog.Filter = "Epitome Files (*.epi)|*.epi|All Files (*.*)|*.*";
			if(mySaveFileDialog.ShowDialog() == DialogResult.OK) // get a filename...
			{
				// make a stream and a formatter, then serialize the EDO document object.
				FileStream s = new FileStream(mySaveFileDialog.FileName, FileMode.OpenOrCreate, FileAccess.Write);
				BinaryFormatter f = new BinaryFormatter();
				// update the UI to show the save path.
				EpitomeFilePathLabel.Text = mySaveFileDialog.FileName;
				try 
				{
					f.Serialize(s,EDO); 
				}
				catch(SerializationException error) 
				{
					MessageBox.Show(
						"Serialization error \"" + error.Message + "\" from " + error.Source,
						"Error Saving Epitome", MessageBoxButtons.OK); 
					EpitomeFilePathLabel.Text = "";
				}
				EpitomeFilePathLabel.Text = mySaveFileDialog.FileName;
			}
			mySaveFileDialog.Filter = filtercache; // what a hack.
		}
		/// <summary>
		/// Load a working set from a .epi file
		/// </summary>
		/// <param name="sender"></param>
		/// <param name="e"></param>
		private void LoadEpitome_Click(object sender, System.EventArgs e)
		{
			string filtercache = myFileOpenDialog.Filter;
			myFileOpenDialog.Filter = "Epitome Files (*.epi)|*.epi|All Files (*.*)|*.*";
			if(myFileOpenDialog.ShowDialog() == DialogResult.OK) // get the filename 
			{
				FileStream s = new FileStream(myFileOpenDialog.FileName, FileMode.Open, FileAccess.Read);
				BinaryFormatter f = new BinaryFormatter();
				try 
				{
					EDO = (EpitomeDocObject)(f.Deserialize(s));
				}
				catch(SerializationException error)
				{
					MessageBox.Show("Deserialization error \"" + error.Message + " from " + error.Source,
						"Error Loading Epitome", MessageBoxButtons.OK);
					EDO = new EpitomeDocObject();
					return;
				}
				// do stuff that happens after an epitome is generated (which loading an epitome is similar to)...
				EpitomeDoneHelper();
				// ...with some exceptions...
				ShowEpitomeSequence.Enabled = false; // this is an exception, since we don't save the sequence.
				// ...and some additional UI field fix-ups.
				ShowInputImage();   // re-display the input image and associated UI text fields.
				EpitomeWidthEntry.Value  = EDO.E.W;	// restore status and parameter entry fields.
				EpitomeHeightEntry.Value = EDO.E.H;
				PatchSizeEntry.Value = EDO.E.PatchSizes[1];
				FixedSizeCheckBox.Checked = (EDO.E.PatchSizes[0] == EDO.E.PatchSizes[1]);
				PixelErrorMethodEntry.SetSelected(PixelErrorMethodEntry.FindStringExact(EDO.E.ErrorMethod), true);
				EpitomeFilePathLabel.Text = myFileOpenDialog.FileName;
				ShowPatches.Enabled = true;
				ShowLatestEpitome.Enabled = true;

				DebugTextBox.AppendText("Loaded Epitome\n");
			}
			myFileOpenDialog.Filter = filtercache;  // what a hack...
		}

		private void GenerateEpitome_Click(object sender, System.EventArgs e)
		{
			// create the epitome object
			EDO.E = new Epitome(EDO.InputImageBitmap,
								(int)EpitomeWidthEntry.Value,
								(int)EpitomeHeightEntry.Value,
								(int)PatchSizeEntry.Value,
								(bool)FixedSizeCheckBox.Checked,
								(int)EpitomeIterationsEntry.Value,
								(int)IterationModulusEntry.Value,
								(string)PixelErrorMethodEntry.SelectedItem,
								DebugPictureBox.Width,
								DebugPictureBox.Height);
			// subscribe to the epitome's events (OnEpitomeString, OnEpitomeBitmap, OnEpitomeDone)
			EDO.E.OnEpitomeString += new Epitome.EpitomeStringEvent(UpdateDebugTextBox);
			EDO.E.OnEpitomeBitmap += new Epitome.EpitomeBitmapEvent(UpdateDebugPictureBox);
			EDO.E.OnEpitomeDone   += new Epitome.EpitomeDoneEvent(UpdateEpitomeDone);

			// update various UI controls for things that can and can't be done during epitome generation.
			GenerateEpitome.Enabled = false;
			ShowEpitomeSequence.Enabled = false;
			ShowPatchDensityMap.Enabled = false;
			MapToImage.Enabled = false;
			ShowPatches.Enabled = true;
			ShowLatestEpitome.Enabled = true;
			StopButton.Enabled = true;

			EDO.IsEpitomeDone = false; // Note that generation is underway.  The OnEpitomeDone event will clear this.

			// start the processing and return to the main UI message pump.
			WorkerThread = new Thread(new ThreadStart(EDO.E.GenerateEpitome));
			WorkerThread.IsBackground = true;
			WorkerThread.Priority = ThreadPriority.Lowest;
			WorkerThread.Start();
		}

		// event handlers for the Epitome object's events
		public void UpdateDebugTextBox(object epitome, Epitome.EpitomeStringArgs e) 
		{
			if(DebugTextBox.InvokeRequired) 
			{
				this.BeginInvoke(new Epitome.EpitomeStringEvent(UpdateDebugTextBox), new object[]{epitome, e});
			} 
			else 
			{
				DebugTextBox.AppendText(e.status);
				DebugTextBox.Focus();
				DebugTextBox.SelectionStart = DebugTextBox.TextLength;
				DebugTextBox.SelectionLength = 0;
				DebugTextBox.ScrollToCaret();
			}
		}
		public void UpdateDebugPictureBox(object epitome, Epitome.EpitomeBitmapArgs e) 
		{
			if(DebugPictureBox.InvokeRequired) 
			{
				this.BeginInvoke(new Epitome.EpitomeBitmapEvent(UpdateDebugPictureBox), new object[]{epitome, e});
			} 
			else 
			{
				DebugShowScaled(e.bitmap);
			}
		}
		public void UpdateEpitomeDone(object epitome, EventArgs e) 
		{
			if(EpitomeImage.InvokeRequired) 
			{
				this.BeginInvoke(new Epitome.EpitomeDoneEvent(UpdateEpitomeDone), new object[]{epitome, e});
			} 
			else 
			{
				EDO.IsEpitomeDone = true;	// Hey, it's done!
				EpitomeDoneHelper();        // contains UI fix-up code that is shared with the load-from-file sequence.
				DebugTextBox.AppendText("Generated Epitome\n");
			}
		}

		// this function is called by the EpitomeDone event handler and after loading an epitome from a file.
		private void EpitomeDoneHelper() 
		{
			ShowEpitomeImage();			// Show the epitome

			// Update various other controls, re-enable various buttons
			EpitomeImageStatsLabel.Text =     // show the epitome image attributes in the UI
				"Epitome Size: " +
				EDO.E.W.ToString() + " x " +
				EDO.E.H.ToString() + " " +
				EDO.E.StatString;

			// update various buttons:	
			GenerateEpitome.Enabled = true;
			ShowEpitomeSequence.Enabled = true; // may have to change if we stop saving epitome sequences.
			ShowPatchDensityMap.Enabled = true;
			SaveEpitome.Enabled = true;
			MapToImage.Enabled = true;
			StopButton.Enabled = false;
		}

		// ask the Epitome object for a bitmap showing some of the patches in isolation.
		private void ShowPatches_Click(object sender, System.EventArgs e)
		{
			DebugShowScaled(EDO.E.GetPatches());
		}

		private void ShowLatestEpitome_Click(object sender, System.EventArgs e)
		{
			DebugShowScaled(EDO.E.GetLatestBitmap());
		}

		private Bitmap ScaleBitmapTo(Bitmap b, int TargetWidth, int TargetHeight) 
		{
			int ratio = Math.Min(TargetWidth  / b.Width, TargetHeight / b.Height);
			int i = 0, j = 0;
			int TempW = b.Width  * ratio;
			int TempH = b.Height * ratio;
			Bitmap Temp = new Bitmap(TempW, TempH);
			for(i = 0; i < TempW; i++)
				for(j = 0; j < TempH; j++) 
				{
					Temp.SetPixel(i,j,b.GetPixel(i/ratio,j/ratio));
				}
			return Temp;
		}

		private void DebugShowScaled(Bitmap b) 
		{
			DebugPictureBox.Image = (Image)(ScaleBitmapTo(b, DebugPictureBox.Width, DebugPictureBox.Height));
		}

		private void ShowEpitomeImage() 
		{
			ShowEpitomeImage(EDO.E.EpitomeBitmap);
		}

		private void ShowEpitomeImage(Bitmap M) 
		{
			if(EDO.EpitomeShowState == "normal")
				EpitomeImage.Image = (Image)M;
			else if(EDO.EpitomeShowState == "tiled") 
			{
				int i = 0, j = 0;
				Bitmap b = new Bitmap(EpitomeImage.Width, EpitomeImage.Height);  // make a tiled bitmap
				for(i = 0; i < EpitomeImage.Width; i++)
					for(j = 0; j < EpitomeImage.Height; j++)
						b.SetPixel(i,j, M.GetPixel(i % M.Width, j % M.Height));
				EpitomeImage.Image = (Image)b;
			} 
			else if(EDO.EpitomeShowState == "scaled") 
			{
				EpitomeImage.Image = (Image)(ScaleBitmapTo(M, EpitomeImage.Width, EpitomeImage.Height));
			}
		}

		private void UpdateEpitomeShowState() 
		{
			if(EpitomeShowNormal.Checked)
				EDO.EpitomeShowState = "normal";
			else if(EpitomeShowTiled.Checked)
				EDO.EpitomeShowState = "tiled";
			else if(EpitomeShowScaled.Checked)
				EDO.EpitomeShowState = "scaled";
			else
				EDO.EpitomeShowState = "error";
			if(EDO.IsEpitomeDone == true) 
			{
				ShowEpitomeImage();
			}
		}

		private void EpitomeShowNormal_CheckedChanged(object sender, System.EventArgs e)
		{
			UpdateEpitomeShowState();
		}

		private void EpitomeShowTiled_CheckedChanged(object sender, System.EventArgs e)
		{
			UpdateEpitomeShowState();
		}

		private void EpitomeShowScaled_CheckedChanged(object sender, System.EventArgs e)
		{
			UpdateEpitomeShowState();
		}

		private void ShowEpitomeSequence_Click(object sender, System.EventArgs e)
		{
			if(EDO.IsEpitomeDone == true) 
			{
				Bitmap temp = new Bitmap(DebugPictureBox.Width, DebugPictureBox.Height);
				int RowStride = (((Bitmap)EDO.E.EpitomeSequence[0]).Height + 1),  // They're all the same size, so [0] is just fine.
					ColStride = (((Bitmap)EDO.E.EpitomeSequence[0]).Width + 1), // The +1s allow one blank pixel between each one.
					NumPerRow = DebugPictureBox.Width / ColStride,
					NumPerCol = DebugPictureBox.Height / RowStride;
			
				for(int i = 0; i < Math.Min(EDO.E.EpitomeSequence.Count, NumPerRow*NumPerCol); i++) // for as many as we have space for
				{
					// determine starting location within temp
					int XStart = (i % NumPerRow) * ColStride,
						YStart = (i / NumPerRow) * RowStride;
					for(int j = 0; j < ((Bitmap)EDO.E.EpitomeSequence[0]).Width; j++)
						for(int k = 0; k < ((Bitmap)EDO.E.EpitomeSequence[0]).Height; k++)
							temp.SetPixel(XStart + j, YStart + k, ((Bitmap)EDO.E.EpitomeSequence[i]).GetPixel(j,k));
				}
				DebugPictureBox.Image = (Image)temp;
			}
		}


		/// <summary>
		/// Ask the epitome to generate a bitmap that maps the epitome back to an image of the original size, 
		/// than show it.
		/// </summary>
		/// <param name="sender"></param>
		/// <param name="e"></param>
		private void MapToImage_Click(object sender, System.EventArgs e)
		{
			InputImage.Image = (Image)EDO.E.MapToImage();  // show the 
			RevertImage.Enabled = true;
		}

		/// <summary>
		/// Re-display the original input image, in case it has been overwritten by MapToImage_Click(). 
		/// </summary>
		/// <param name="sender"></param>
		/// <param name="e"></param>
		private void RevertImage_Click(object sender, System.EventArgs e)
		{
			InputImage.Image = (Image)EDO.InputImageBitmap;
			RevertImage.Enabled = false;
		}

		private void ShowPatchDensityMap_Click(object sender, System.EventArgs e)
		{
			DebugShowScaled(EDO.E.GetPatchDensityMap());
		}

		private void StopButton_Click(object sender, System.EventArgs e)
		{
			EDO.E.Finish();
			StopButton.Enabled = false;
		}

		private void OnResize(object sender, System.EventArgs e)
		{
			if(EDO.InputImagePath != null)	// this is only necessary if we get a resize event after the user
			{								// has loaded an input image.
				ShowInputImage();
			}
		}

		/// <summary>
		/// When the user clicks in the epitome image, get the list of patches at that spot, and update the 
		/// InputImage bitmap to display the source locations of all those patches.
		/// </summary>
		/// <param name="sender"></param>
		/// <param name="e"></param>
		private void EpitomeImage_MouseDown(object sender, System.Windows.Forms.MouseEventArgs e)
		{
			if(!EDO.IsEpitomeDone) return;		// can't do this until an epitome is finished.
			int X = e.X, Y = e.Y;
			if(EDO.EpitomeShowState == "tiled") // correct the click coordinates respective of the display mode
			{
				X = X % EDO.E.W;
				Y = Y % EDO.E.H;
			}
			if(EDO.EpitomeShowState == "scaled") 
			{
				int ratio = Math.Min(EpitomeImage.Width  / EDO.E.W, EpitomeImage.Height / EDO.E.H);
				X /= ratio;
				Y /= ratio;
			}
			if(X >= EDO.E.W || Y >= EDO.E.H) // coordinates still too big?  User clicked outside the epitome bitmap.
				return;

			Bitmap b = new Bitmap(InputImage.Image);
			
			foreach(int p in EDO.E.EpitomePatchMap[X,Y]) 
			{
				Patch P = (Patch)(EDO.E.Patches[p]);
				int xw = P.OriginX + P.Width -1,	// locations for the "far" edges of the patch
					yh = P.OriginY + P.Height -1;
				for(int x = P.OriginX; x < P.OriginX + P.Width; x++)
				{
					b.SetPixel(x,P.OriginY,Hilight(b.GetPixel(x,P.OriginY)));
					b.SetPixel(x,yh,       Hilight(b.GetPixel(x,yh)));
				}
				for(int y = P.OriginY; y < P.OriginY + P.Height; y++) 
				{
					b.SetPixel(P.OriginX,y,Hilight(b.GetPixel(P.OriginX,y))); 
					b.SetPixel(xw,       y,Hilight(b.GetPixel(xw,       y))); 
				}
			}
			InputImage.Image = (Image)b;
			RevertImage.Enabled = true;
		}

		/// <summary>
		/// When the user clicks in the source image, get the list of all patches at that spot and mark the
		/// epitome bitmap image to show the destination locations of all those patches.
		/// </summary>
		/// <param name="sender"></param>
		/// <param name="e"></param>
		private void InputImage_MouseDown(object sender, System.Windows.Forms.MouseEventArgs e)
		{
			if(!EDO.IsEpitomeDone) return;		// can't do this until an epitome is finished.
			int X = e.X, Y = e.Y;
			// possibly correct X and Y for scaling of the source image.
			if(InputImage.SizeMode == PictureBoxSizeMode.StretchImage) 
			{
				double xRatio = (double)InputImage.Width  / (double)EDO.InputImageBitmap.Width;
				double yRatio = (double)InputImage.Height / (double)EDO.InputImageBitmap.Height;
				X = (int)((double)X / xRatio);
				Y = (int)((double)Y / yRatio);
			}
			if(X >= EDO.InputImageBitmap.Width || Y >= EDO.InputImageBitmap.Height) // user clicked outside the image
				return;
			Bitmap b = new Bitmap(EDO.E.LatestBitmap);
			foreach(int p in EDO.E.SourcePatchMap[X,Y]) 
			{
				Patch P = (Patch)(EDO.E.Patches[p]);
				int xw = (P.DestinationX + P.Width -1) % EDO.E.W,	// wrap-safe locations for the "far" edges of the patch
					yh = (P.DestinationY + P.Height -1) % EDO.E.H;
				for(int x = P.DestinationX; x < P.DestinationX + P.Width; x++) 
				{
					int x0 = x % EDO.E.W;
					b.SetPixel(x0,P.DestinationY, Hilight(b.GetPixel(x0,P.DestinationY)));
					b.SetPixel(x0,yh,             Hilight(b.GetPixel(x0,yh)));
				}
				for(int y = P.DestinationY; y < P.DestinationY + P.Height; y++) 
				{
					int y0 = y % EDO.E.H;
					b.SetPixel(P.DestinationX,y0, Hilight(b.GetPixel(P.DestinationX,y0)));
					b.SetPixel(xw,            y0, Hilight(b.GetPixel(xw,            y0)));
				}
			}
			ShowEpitomeImage(b);
		}

		private Color Hilight(Color C)					
		{
			return Color.LimeGreen;
		}

	}
}
