using System;
using System.Drawing;
using System.Collections;
using System.Runtime.Serialization;
using System.Runtime.Serialization.Formatters.Binary;

namespace epitome
{
	/// <summary>
	/// A patch is a rectangular bitmap shared by a source image and an epitome image.  A patch consists of:
	/// * the patch's origin in the source image,
	/// * the patch's destination in the epitome image,
	/// * the patch's size,
	/// * pixel data copied from the source image
	/// </summary>
	[Serializable]
	public class Patch:System.Object
	{
		/// <summary>
		/// Top-left location of this patch in the source image:
		/// </summary>
		public int OriginX;
		public int OriginY;
		/// <summary>
		/// Top-left location of this patch in the epitome image
		/// </summary>
		public int DestinationX;
		public int DestinationY;
		/// <summary>
		/// Current best score for the patch
		/// </summary>
		public double CurScore;
        /// <summary>
		/// Size of the patch
		/// </summary>
		public int Width;
		public int Height;
		/// <summary>
		/// Bitmap data copied from source image
		/// </summary>
		public Bitmap PixelData;

		public Patch()
		{
		}
		public Patch(Bitmap Source, int PatchX, int PatchY, int W, int H) 
		{
			OriginX = PatchX;
			OriginY = PatchY;
			Width = W;
			Height = H;
			DestinationX = 0;
			DestinationY = 0;

			PixelData = new Bitmap(Width, Height);  // create the patch bitmap and copy its pixels from the source image
			for(int i = 0; i < Width; i++)
				for(int j = 0; j < Height; j++)
					PixelData.SetPixel(i,j,
						Source.GetPixel((PatchX + i) % Source.Width, (PatchY + j) % Source.Height) );
		}

		public override String ToString() // returns strings of the form "[X,Y] @ WxH"; e.g. "[12x17] @ 8x8"
		{
			return "[" + OriginX.ToString() + "," + OriginY.ToString() + "] @ " + Width.ToString() + "x" + Height.ToString();
		}

	}
}
